function [a, b, c] = find_abc(t_Peak_Desired, PulseWidth_Desired, PulseAmplitude_Desired, Pulse_Off_Desired, use_Parallel)

% Finds the variables a, b, and c for the following equation based on desired parameters:

% y(x) = c*(exp(-a*x)-exp(-b*x))

% t_Peak_Desired: time at which Y is max
% PulseWidth_Desired: time at which Y decays to the Pulse Off limit
% PulseAmplitude_Desired: Amplitude of maximum Y value
% Pulse_Off_Desired: Pulse off limit set as percentage of maximum
% amplitude - possible range: 0 - 1
% use_Parallel: 1=use parfor; 0=do not use parfor


% If nothing is plotted, t_peak is likely too close to pulse width. Reduce
% t_peak or increase pulse width

b_Low = 0.000001;
b_High = 10000;
b_NumPoints = 100;

a_Low = b_Low*.99999999;
a_High = b_High*.999999999;
a_NumPoints = 100;

b_Values = logspace(-8,6,b_NumPoints);
a_Values = .999999*b_Values;

clear a_Low a_High b_Low b_High

Parameters = zeros(a_NumPoints*b_NumPoints,2);
Results = zeros(a_NumPoints*b_NumPoints,2);

n = 1;
for i = 1:size(a_Values,2)
    for j = 1:size(b_Values,2)
        Parameters(n,1:2) = [a_Values(i) b_Values(j)];
        n=n+1;
    end
end

switch use_Parallel
    case 0
        for i = 1:size(Parameters,1)
            a = Parameters(i,1);
            b = Parameters(i,2);
            t_PeakActual = t_PeakActual_Calc(a,b);
            PulseOff_Value = Discharge(a,b,1,t_PeakActual)*Pulse_Off_Desired;
            t_Off_Actual = t_Off_Actual_Calc(a,b,1,t_PeakActual,PulseOff_Value);
            Results(i,:) = [t_PeakActual, t_Off_Actual];
        end
    case 1
        parfor i = 1:size(Parameters,1)
            a = Parameters(i,1);
            b = Parameters(i,2);
            t_PeakActual = t_PeakActual_Calc(a,b);
            PulseOff_Value = Discharge(a,b,1,t_PeakActual)*Pulse_Off_Desired;
            t_Off_Actual = t_Off_Actual_Calc(a,b,1,t_PeakActual,PulseOff_Value);
            Results(i,:) = [t_PeakActual, t_Off_Actual];
        end
end

clear Parameters

x = a_Values;
y = b_Values;
[X,Y] = meshgrid(x,y);
clear x y

Z_t_Peak = zeros(a_NumPoints,b_NumPoints);
Z_PulseWidth = zeros(a_NumPoints,b_NumPoints);

for i = 1:a_NumPoints
    for j = 1:b_NumPoints
        if isnan(Results(j+(i-1)*a_NumPoints,1)) | i>j
            Z_t_Peak(j,i) = nan;
            Z_PulseWidth(j,i) = nan;
        else
            Z_t_Peak(j,i) = Results(j+(i-1)*a_NumPoints,1);
            Z_PulseWidth(j,i) = Results(j+(i-1)*a_NumPoints,2);
        end
    end
end

[ab_t_Peak, contour_t_Peak] = contour(X,Y,Z_t_Peak,[t_Peak_Desired t_Peak_Desired],'-b');
hold on
[ab_PulseWidth, contour_PulseWidth] = contour(X,Y,Z_PulseWidth,[PulseWidth_Desired PulseWidth_Desired],'-r');
close all

clear X Y Z_PulseWidth Z_t_Peak

ab_t_Peak = ab_t_Peak(:,ab_t_Peak(1,:)<ab_t_Peak(2,:))';
ab_PulseWidth = ab_PulseWidth(:,ab_PulseWidth(1,:)<ab_PulseWidth(2,:))';

ab_t_Peak = ab_t_Peak(2:end,:);
ab_PulseWidth = ab_PulseWidth(2:end,:);

[a, b] = IntersectionFunctions(ab_t_Peak, ab_PulseWidth);

clear ab_t_Peak ab_PulseWidth

PulseAmplitudeActual = Discharge(a,b,1,t_PeakActual_Calc(a,b));
c = PulseAmplitude_Desired/PulseAmplitudeActual;

t_PeakActual = t_PeakActual_Calc(a,b);
PulseOff_Value = PulseAmplitude_Desired*Pulse_Off_Desired;
t_Off_Actual = t_Off_Actual_Calc(a,b,c,t_PeakActual,PulseOff_Value);

x = linspace(0,t_Off_Actual,500);
y = Discharge(a,b,c,x);

figure('Name','Relationship Plot','Units','pixels','Position',[50 50 1000 500])
hold on
plot(x,y,'-r')
plot([0,t_Off_Actual],[Pulse_Off_Desired*PulseAmplitude_Desired, ...
    Pulse_Off_Desired*PulseAmplitude_Desired],'--b')
plot([t_PeakActual t_PeakActual],[0 PulseAmplitude_Desired],'-b')
xlabel('Time (s)')
ylabel('Voltage (V)')
legend('Discharge Function','V_{off}','t_{peak}')
end
%%
function [a, b] = IntersectionFunctions(ab_t_Peak, ab_PulseWidth)

a_Values_t_peak = ab_t_Peak(:,1);
b_Values_t_peak = ab_t_Peak(:,2);

a_Values_PulseWidth = ab_PulseWidth(:,1);
b_Values_PulseWidth = ab_PulseWidth(:,2);

[a, b] = polyxpoly(a_Values_t_peak,b_Values_t_peak,a_Values_PulseWidth,b_Values_PulseWidth);

if isempty(a) | isempty(b)
    a = 0;
    b = 0;
end
end
%%
function [y] = Discharge(a,b,c,t)
y = c*(exp(-a*t)-exp(-b*t));
end
%%
function [t_PeakActual] = t_PeakActual_Calc(a,b)
t_PeakActual = log(a/b)/(a-b);
end
%%
function [t_Off_Actual] = t_Off_Actual_Calc(a,b,c,t_PeakActual,PulseOff_Value)
x = [t_PeakActual t_PeakActual*2];
[Amp_Error, x_int] = Amp_Error_Calc(a,b,c,x,t_PeakActual,PulseOff_Value);

if Amp_Error == 0 & x_int == 0
    t_Off_Actual = 0;
else
    while Amp_Error > 0.000001
        x(end+1) = x_int;
        [Amp_Error, x_int] = Amp_Error_Calc(a,b,c,x,t_PeakActual,PulseOff_Value);
    end
    t_Off_Actual = x_int;
end
end
%%
function [Amp_Error, x_int] = Amp_Error_Calc(a,b,c,x,t_PeakActual,PulseOff_Value)

NoSolution = 0;

if a>b | a==b
    Amp_Error = 0;
    x_int = 0;
elseif size(x,1)<3
    x_check = interp1(Discharge(a,b,c,x),x,PulseOff_Value,"linear",'extrap');
    while x_check < 0 | x_check < t_PeakActual
        x(end+1) = x(end)*2;
        if ~isempty(find(Discharge(a,b,c,x) == 0,1))
            NoSolution = 1;
            break
        end
        x_check = interp1(Discharge(a,b,c,x),x,PulseOff_Value,"linear",'extrap');
    end
    if NoSolution == 1
        Amp_Error = 0;
        x_int = 0;
    else
        x_int = interp1(Discharge(a,b,c,x),x,PulseOff_Value,"linear",'extrap');
        y_Actual = Discharge(a,b,c,x_int);
        Amp_Error = (y_Actual-PulseOff_Value)/PulseOff_Value;
    end
else
    x_check = interp1(Discharge(a,b,c,x),x,PulseOff_Value,"spline",'extrap');
    while x_check<0 | x_check < t_PeakActual
        x(end+1) = x(end)*2;
        if ~isempty(find(Discharge(a,b,c,x) == 0,1))
            NoSolution = 1;
            break
        end
        x_check = interp1(Discharge(a,b,c,x),x,PulseOff_Value,"spline",'extrap');
    end
    if NoSolution == 1
        Amp_Error = 0;
        x_int = 0;
    else
        x_int = interp1(Discharge(a,b,c,x),x,PulseOff_Value,"spline",'extrap');
        y_Actual = Discharge(a,b,c,x_int);
        Amp_Error = (y_Actual-PulseOff_Value)/PulseOff_Value;
    end
end
end